@@ -8,6 +8,7 @@
 
 #include "device/Device.h"
 #include "field/ISurfaceVolumeField.h"
+#include "field/FieldStatistics.h"
 #include "directional/ISurfaceSamplingDistribution.h"
 #include "directional/IVolumeSamplingDistribution.h"
 
@@ -79,19 +80,19 @@ typedef ISurfaceVolumeField IGuidingField;
 
 #define OPENPGL_FIELD_STRING "OPENPGL_" OPENPGL_VERSION_STRING "_FIELD"
 
-extern "C" OPENPGL_DLLEXPORT PGLDevice pglNewDevice(PGL_DEVICE_TYPE deviceType)OPENPGL_CATCH_BEGIN
+extern "C" OPENPGL_DLLEXPORT PGLDevice pglNewDevice(PGL_DEVICE_TYPE deviceType, size_t numThreads)OPENPGL_CATCH_BEGIN
 {
 #ifdef OPENPGL_DEVICE_TYPE_CPU_4
     if (deviceType == PGL_DEVICE_TYPE_CPU_4)
-        return (PGLDevice) newDeviceCPU4();
+        return (PGLDevice) newDeviceCPU4(numThreads);
 #endif
 #ifdef OPENPGL_DEVICE_TYPE_CPU_8
     if (deviceType == PGL_DEVICE_TYPE_CPU_8)
-        return (PGLDevice) newDeviceCPU8();
+        return (PGLDevice) newDeviceCPU8(numThreads);
 #endif
 #ifdef OPENPGL_DEVICE_TYPE_CPU_16
     if (deviceType == PGL_DEVICE_TYPE_CPU_16)
-        return (PGLDevice) newDeviceCPU16();
+        return (PGLDevice) newDeviceCPU16(numThreads);
 #endif
 
     throw std::runtime_error("invalid vectorSize parameter!");
@@ -251,6 +252,25 @@ extern "C" OPENPGL_DLLEXPORT  bool pglFieldValidate(PGLField field)
     return gField->validate(true, true);
 }
 
+extern "C" OPENPGL_DLLEXPORT bool pglFieldCompare(PGLField fieldA, PGLField fieldB)
+{
+    auto *gFieldA = (const IGuidingField *)fieldA;
+    auto *gFieldB = (const IGuidingField *)fieldB;
+    return gFieldA->operator==(gFieldB);
+}
+
+extern "C" OPENPGL_DLLEXPORT PGLFieldStatistics pglFieldGetSurfaceStatistics(PGLField field)
+{
+    const auto *gField = (const IGuidingField *)field;
+    return (PGLFieldStatistics)gField->getSurfaceStatistics();
+}
+
+extern "C" OPENPGL_DLLEXPORT PGLFieldStatistics pglFieldGetVolumeStatistics(PGLField field)
+{
+    const auto *gField = (const IGuidingField *)field;
+    return (PGLFieldStatistics)gField->getVolumeStatistics();
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // Region /////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
@@ -373,6 +393,18 @@ extern "C" OPENPGL_DLLEXPORT PGLSampleData pglSampleStorageGetSampleVolume(PGLSa
     return gSampleStorage->getSampleVolume(idx);
 }
 
+extern "C" OPENPGL_DLLEXPORT bool pglSampleStorageValidate(PGLSampleStorage sampleStorage)
+{
+    auto *gSampleStorage = (openpgl::SampleDataStorage *)sampleStorage;
+    return gSampleStorage->validate();
+}
+
+extern "C" OPENPGL_DLLEXPORT bool pglSampleStorageCompare(PGLSampleStorage sampleStorageA, PGLSampleStorage sampleStorageB)
+{
+    auto *gSampleStorageA = (openpgl::SampleDataStorage *)sampleStorageA;
+    auto *gSampleStorageB = (openpgl::SampleDataStorage *)sampleStorageB;
+    return gSampleStorageA->operator==(*gSampleStorageB);
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // SampleData /////////////////////////////////////////////////////////////////
@@ -609,6 +641,12 @@ extern "C" OPENPGL_DLLEXPORT float pglSurfaceSamplingDistributionIncomingRadianc
     return gSurfaceSamplingDistribution->pdfLi(openpgl::Vector3(direction.x, direction.y, direction.z));
 }
 
+extern "C" OPENPGL_DLLEXPORT uint32_t pglSurfaceSamplingDistributionGetId(PGLSurfaceSamplingDistribution surfaceSamplingDistribution)
+{
+    ISurfaceSamplingDistribution* gSurfaceSamplingDistribution =  (ISurfaceSamplingDistribution*)surfaceSamplingDistribution;
+    return gSurfaceSamplingDistribution->getId();
+}
+
 extern "C" OPENPGL_DLLEXPORT bool pglSurfaceSamplingDistributionValidate(PGLSurfaceSamplingDistribution surfaceSamplingDistribution)
 {
     ISurfaceSamplingDistribution* gSurfaceSamplingDistribution =  (ISurfaceSamplingDistribution*)surfaceSamplingDistribution;
@@ -683,6 +721,12 @@ extern "C" OPENPGL_DLLEXPORT float pglVolumeSamplingDistributionIncomingRadiance
     return gVolumeSamplingDistribution->pdfLi(openpgl::Vector3(direction.x, direction.y, direction.z));
 }
 
+extern "C" OPENPGL_DLLEXPORT uint32_t pglVolumeSamplingDistributionGetId(PGLVolumeSamplingDistribution volumeSamplingDistribution)
+{
+    IVolumeSamplingDistribution* gVolumeSamplingDistribution =  (IVolumeSamplingDistribution*)volumeSamplingDistribution;
+    return gVolumeSamplingDistribution->getId();
+}
+
 extern "C" OPENPGL_DLLEXPORT bool pglVolumeSamplingDistributionValidate(PGLVolumeSamplingDistribution volumeSamplingDistribution)
 {
     IVolumeSamplingDistribution* gVolumeSamplingDistribution =  (IVolumeSamplingDistribution*)volumeSamplingDistribution;
@@ -708,7 +752,7 @@ extern "C" OPENPGL_DLLEXPORT bool pglVolumeSamplingDistributionSupportsApplySing
     return gVolumeSamplingDistribution->supportsApplySingleLobeHenyeyGreensteinProduct();
 }
 
-extern "C" OPENPGL_DLLEXPORT void pglFieldArgumentsSetDefaults(PGLFieldArguments &fieldArguments, const PGL_SPATIAL_STRUCTURE_TYPE spatialType, const PGL_DIRECTIONAL_DISTRIBUTION_TYPE directionalType)
+extern "C" OPENPGL_DLLEXPORT void pglFieldArgumentsSetDefaults(PGLFieldArguments &fieldArguments, const PGL_SPATIAL_STRUCTURE_TYPE spatialType, const PGL_DIRECTIONAL_DISTRIBUTION_TYPE directionalType, const bool deterministic, const size_t maxSamplesPerLeaf)
 {
     switch (spatialType)
     {
@@ -716,17 +760,19 @@ extern "C" OPENPGL_DLLEXPORT void pglFieldArgumentsSetDefaults(PGLFieldArguments
     case PGL_SPATIAL_STRUCTURE_TYPE::PGL_SPATIAL_STRUCTURE_KDTREE:
         fieldArguments.spatialStructureType = PGL_SPATIAL_STRUCTURE_KDTREE;
         fieldArguments.spatialSturctureArguments = new PGLKDTreeArguments();
+        reinterpret_cast<PGLKDTreeArguments *>(fieldArguments.spatialSturctureArguments)->maxSamples = maxSamplesPerLeaf;
         break;
     }
 
-    fieldArguments.deterministic = false;
-
+    fieldArguments.deterministic = deterministic;
+    fieldArguments.debugArguments.fitRegions = true;
+    
     switch (directionalType)
     {
     default:
     case PGL_DIRECTIONAL_DISTRIBUTION_TYPE::PGL_DIRECTIONAL_DISTRIBUTION_PARALLAX_AWARE_VMM:
         fieldArguments.directionalDistributionType = PGL_DIRECTIONAL_DISTRIBUTION_PARALLAX_AWARE_VMM;
-        fieldArguments.directionalDistributionArguments = new PGLVMMFactoryArguments();
+        fieldArguments.directionalDistributionArguments = new PGLVMMFactoryArguments(maxSamplesPerLeaf);
         break;
     case PGL_DIRECTIONAL_DISTRIBUTION_TYPE::PGL_DIRECTIONAL_DISTRIBUTION_QUADTREE:
         fieldArguments.directionalDistributionType = PGL_DIRECTIONAL_DISTRIBUTION_QUADTREE;
@@ -734,10 +780,68 @@ extern "C" OPENPGL_DLLEXPORT void pglFieldArgumentsSetDefaults(PGLFieldArguments
         break;
     case PGL_DIRECTIONAL_DISTRIBUTION_TYPE::PGL_DIRECTIONAL_DISTRIBUTION_VMM:
         fieldArguments.directionalDistributionType = PGL_DIRECTIONAL_DISTRIBUTION_VMM;
-        fieldArguments.directionalDistributionArguments = new PGLVMMFactoryArguments();
+        fieldArguments.directionalDistributionArguments = new PGLVMMFactoryArguments(maxSamplesPerLeaf);
         break;
     }
 
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// FieldStatistics ////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+
+extern "C" OPENPGL_DLLEXPORT void pglReleaseFieldStatistics(PGLFieldStatistics fieldStatistics)
+{
+    auto *gFieldStatistics = (openpgl::FieldStatistics *)fieldStatistics;
+    delete gFieldStatistics;
 
 }
 
+extern "C" OPENPGL_DLLEXPORT PGLString pglFieldStatisticsToString(PGLFieldStatistics fieldStatistics)
+{
+    auto *gFieldStatistics = (openpgl::FieldStatistics *)fieldStatistics;
+    std::string str = gFieldStatistics->toString();
+    PGLString pglStr;
+    pglStr.m_size = str.length()+1;
+    pglStr.m_str = new char[pglStr.m_size];
+    strcpy(pglStr.m_str, str.c_str());
+    return pglStr;
+}
+
+extern "C" OPENPGL_DLLEXPORT PGLString pglFieldStatisticsHeaderCSVString(PGLFieldStatistics fieldStatistics)
+{
+    auto *gFieldStatistics = (openpgl::FieldStatistics *)fieldStatistics;
+    std::string str = gFieldStatistics->headerCSVString();
+    PGLString pglStr;
+    pglStr.m_size = str.length()+1;
+    pglStr.m_str = new char[pglStr.m_size];
+    strcpy(pglStr.m_str, str.c_str());
+    return pglStr;
+}
+
+extern "C" OPENPGL_DLLEXPORT PGLString pglFieldStatisticsToCSVString(PGLFieldStatistics fieldStatistics)
+{
+    auto *gFieldStatistics = (openpgl::FieldStatistics *)fieldStatistics;
+    std::string str = gFieldStatistics->toCSVString();
+    PGLString pglStr;
+    pglStr.m_size = str.length()+1;
+    pglStr.m_str = new char[pglStr.m_size];
+    strcpy(pglStr.m_str, str.c_str());
+    return pglStr;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// String /////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+
+extern "C" OPENPGL_DLLEXPORT void pglReleaseString(PGLString str)
+{
+    if (str.m_str)
+    {
+        delete[] str.m_str;
+        str.m_str = nullptr;
+        str.m_size = 0;
+    }
+}
+
