@@ -35,12 +35,17 @@ struct ISurfaceSamplingDistribution
 
     virtual std::string toString() const = 0;
 
+    uint32_t getId() const { return m_id; };
+
+    void setId(const uint32_t id) { m_id = id; };
+
     virtual void setRegion(const IRegion* region) = 0;
 
     virtual const IRegion* getRegion() const = 0;
 
 protected:
     //const IRegion* m_region {nullptr};
+    uint32_t m_id {0};
 };
 
 }
\ No newline at end of file
