@@ -7,12 +7,13 @@
 
 #include <tbb/concurrent_vector.h>
 
+#include "KDTreeStatistics.h"
 
 #include <fstream>
 #include <string>
 #include <iostream>
 
-//#define MERGE_SPLITDIM_AND_NODE_IDX
+#define USE_TREELETS
 
 namespace openpgl
 {
@@ -27,27 +28,17 @@ struct KDNode
     };
 
     float splitPosition {0.0f};
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
     uint32_t splitDimAndNodeIdx{0};
-#else
-    uint8_t splitDim{0};
-    uint32_t nodeIdx{0};
-#endif
+
     /////////////////////////////
     // Child node functions
     /////////////////////////////
     bool isChild() const
     {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         return ( splitDimAndNodeIdx >> 30) < ELeafNode;
-#else
-    return splitDim < ELeafNode;
-#endif
-
     }
 
     void setLeftChildIdx(const uint32_t &idx) {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT(idx < (1U<<31));
         OPENPGL_ASSERT( (splitDimAndNodeIdx & (3U << 30)) != (3U<<30));
         OPENPGL_ASSERT( (splitDimAndNodeIdx >> 30)  != 3);
@@ -55,22 +46,14 @@ struct KDNode
 
         splitDimAndNodeIdx = ((splitDimAndNodeIdx >> 30) << 30)|idx;
         OPENPGL_ASSERT( idx == getLeftChildIdx());
-#else
-    splitDim = ELeafNode;
-    nodeIdx = idx;
-#endif
     }
 
     uint32_t getLeftChildIdx() const {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT( (splitDimAndNodeIdx & (3U << 30)) != (3U<<30));
         OPENPGL_ASSERT( (splitDimAndNodeIdx >> 30)  != 3);
         OPENPGL_ASSERT( !isLeaf() );
 
         return (splitDimAndNodeIdx << 2) >> 2;
-#else
-        return  nodeIdx;
-#endif
         }
 
     /////////////////////////////
@@ -79,10 +62,10 @@ struct KDNode
 
     void setToInnerNode( const uint8_t &_splitDim, const float &_splitPos, const uint32_t &_leftChildIdx) {
         splitPosition = _splitPos;
-        splitDim = _splitDim;
-        nodeIdx = _leftChildIdx;
-        //setSplitDim(splitDim);
-        //setLeftChildIdx(leftChildIdx);
+        splitDimAndNodeIdx = 0;
+        splitDimAndNodeIdx = (uint32_t(_splitDim)<<30);
+        splitDimAndNodeIdx = ((splitDimAndNodeIdx >> 30) << 30)|_leftChildIdx;
+
         OPENPGL_ASSERT(_splitDim == getSplitDim());
         OPENPGL_ASSERT(_leftChildIdx == getLeftChildIdx());
     }
@@ -93,24 +76,15 @@ struct KDNode
     /////////////////////////////
 
     void setLeaf(){
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         splitDimAndNodeIdx = (3U<<30);
         OPENPGL_ASSERT(isLeaf());
-#else
-        splitDim = ELeafNode;
-#endif
     }
 
     bool isLeaf() const {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         return (splitDimAndNodeIdx >> 30) == 3;
-#else
-        return splitDim == ELeafNode;
-#endif
     }
 
     void setChildNodeIdx(const uint32_t &idx) {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT(idx < (1U<<31));
         OPENPGL_ASSERT( (splitDimAndNodeIdx & (3U << 30)) != (3U<<30));
         OPENPGL_ASSERT( (splitDimAndNodeIdx >> 30)  != 3);
@@ -118,33 +92,21 @@ struct KDNode
 
         splitDimAndNodeIdx = ((splitDimAndNodeIdx >> 30) << 30)|idx;
         OPENPGL_ASSERT( idx == getLeftChildIdx());
-#else
-        nodeIdx = idx;
-#endif
     }
 
     void setDataNodeIdx(const uint32_t &idx)
     {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT(idx < (1U<<31)); // checks if the idx is in the right range
         setLeaf();
         splitDimAndNodeIdx = ((splitDimAndNodeIdx >> 30) << 30)|idx;
         OPENPGL_ASSERT( isLeaf());
         OPENPGL_ASSERT( getDataIdx() == idx);
-#else
-        setLeaf();
-        nodeIdx = idx;
-#endif
     }
 
     uint32_t getDataIdx() const
     {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT(isLeaf());
         return (splitDimAndNodeIdx << 2) >> 2;
-#else
-    return nodeIdx;
-#endif
     }
 
 
@@ -154,22 +116,13 @@ struct KDNode
 
     uint8_t getSplitDim() const
     {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
-        OPENPGL_ASSERT((splitDimAndNodeIdx >> 30) < ELeafNode);
         return (splitDimAndNodeIdx >> 30);
-#else
-        return splitDim;
-#endif
     }
 
     void setSplitDim(const uint8_t &splitAxis) {
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         OPENPGL_ASSERT(splitAxis<ELeafNode);
         splitDimAndNodeIdx = (uint32_t(splitAxis)<<30);
         OPENPGL_ASSERT (splitAxis == getSplitDim());
-#else
-        splitDim = splitAxis;
-#endif
     }
 
     float getSplitPivot() const {
@@ -183,25 +136,42 @@ struct KDNode
     void serialize(std::ostream& stream)const
     {
         stream.write(reinterpret_cast<const char*>(&splitPosition), sizeof(float));
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         stream.write(reinterpret_cast<const char*>(&splitDimAndNodeIdx), sizeof(uint32_t));
-#else
-        stream.write(reinterpret_cast<const char*>(&splitDim), sizeof(uint8_t));
-        stream.write(reinterpret_cast<const char*>(&nodeIdx), sizeof(uint32_t));
-#endif
     }
 
     void deserialize(std::istream& stream)
     {
         stream.read(reinterpret_cast<char*>(&splitPosition), sizeof(float));
-#ifdef MERGE_SPLITDIM_AND_NODE_IDX
         stream.read(reinterpret_cast<char*>(&splitDimAndNodeIdx), sizeof(uint32_t));
-#else
-        stream.read(reinterpret_cast<char*>(&splitDim), sizeof(uint8_t));
-        stream.read(reinterpret_cast<char*>(&nodeIdx), sizeof(uint32_t));
-#endif
     }
 
+    bool operator==(const KDNode& b) const {
+        bool equal = true;
+        if (splitPosition != b.splitPosition || isLeaf() != b.isLeaf()) 
+        {
+            equal = false;
+        }
+        if (!isLeaf())
+        {
+            if(getSplitDim() != b.getSplitDim() || 
+                getLeftChildIdx() != b.getLeftChildIdx())
+            {
+                equal = false;
+            }
+        } else {
+            if (getDataIdx() != b.getDataIdx()) 
+            {
+                equal = false;
+            }
+        }
+        return equal;
+    }
+
+};
+
+struct KDTreeLet
+{
+    KDNode nodes[8];
 };
 
 
@@ -210,10 +180,18 @@ struct KDTree
 {
     KDTree() = default;
 
+    KDTree(const KDTree&) = delete;
+
     ~KDTree()
     {
         if(m_nodesPtr)
             delete[] m_nodesPtr;
+
+        if(m_treeLets){
+            delete[] m_treeLets;
+            m_treeLets = nullptr;
+            m_numTreeLets = 0;
+        }
     }
 
     inline void init(const BBox &bounds, size_t numNodesReseve = 0)
@@ -255,7 +233,6 @@ struct KDTree
 
     const KDNode& getNode( const size_t &idx ) const
     {
-        //OPENPGL_ASSERT( m_isInit );
         OPENPGL_ASSERT( m_nodes.size() > idx );
         return m_nodes[idx];
     }
@@ -276,6 +253,35 @@ struct KDTree
         return m_bounds;
     }
 
+    void rearrangeNodeForCompare(const KDNode& node, int idx, std::vector<KDNode>& newNodes, std::vector< uint32_t > &dataStorageIndices) const {
+        if (!node.isLeaf())
+        {
+            uint32_t leftIdx = node.getLeftChildIdx();
+            KDNode nodeLeft = m_nodes[leftIdx];
+            KDNode nodeRight = m_nodes[leftIdx+1];
+            uint32_t newIdx = newNodes.size();
+            newNodes[idx].setLeftChildIdx(newIdx);
+            newNodes.push_back(nodeLeft);
+            newNodes.push_back(nodeRight);
+            rearrangeNodeForCompare(nodeLeft, newIdx, newNodes, dataStorageIndices);
+            rearrangeNodeForCompare(nodeRight, newIdx+1, newNodes, dataStorageIndices);
+        } else {
+            uint32_t dataIdx = node.getDataIdx();
+            uint32_t newDataIdx = dataStorageIndices.size();
+            dataStorageIndices.push_back(dataIdx);
+            newNodes[idx].setDataNodeIdx(newDataIdx);
+        }
+    }
+
+    void rearrangeNodesForCompare(std::vector<KDNode> &newNodes, std::vector< uint32_t > &dataStorageIndices) const
+    {
+        if (m_nodes.size() > 0) {
+            KDNode root = m_nodes[0];
+            newNodes.push_back(root);
+            rearrangeNodeForCompare(root, 0, newNodes, dataStorageIndices);
+        }
+    }
+
     void finalize()
     {
         if(m_nodesPtr){
@@ -290,9 +296,87 @@ struct KDTree
             {
                 m_nodesPtr[n] = m_nodes[n];
             }
-        }       
+#ifdef USE_TREELETS
+            buildTreeLets();
+#endif
+        }     
     }
 
+    void buildTreeLets()
+    {
+        if(m_treeLets){
+            delete[] m_treeLets;
+            m_treeLets = nullptr;
+            m_numTreeLets = 0;
+        }
+
+        std::vector<KDTreeLet> treeLets;
+        treeLets.push_back(KDTreeLet());
+        insertNode(m_nodesPtr[0], 0, 0, 0, treeLets);
+
+        m_numTreeLets = treeLets.size();
+        m_treeLets = new KDTreeLet[m_numTreeLets];
+        for ( int i = 0; i < m_numTreeLets; i++){
+            m_treeLets[i] = treeLets[i];
+        }
+    }
+
+    uint32_t insertNode(const KDNode& node, uint32_t nodeIdx, uint32_t treeLetIdx, uint32_t treeDepth, std::vector<KDTreeLet>& treeLets) {
+        uint32_t treeLetLevel = treeDepth % 3;
+        uint32_t globalNodeId = (treeLetIdx * 8) + nodeIdx;
+        treeLets[treeLetIdx].nodes[nodeIdx] = node;
+        if (!node.isLeaf()) {
+            if(treeLetLevel == 0) {
+                uint32_t childIdx = node.getLeftChildIdx();
+                uint32_t newChildIdx = insertNode(m_nodesPtr[childIdx], 1, treeLetIdx, treeDepth+1,treeLets);
+                insertNode(m_nodesPtr[childIdx+1], 2, treeLetIdx, treeDepth+1,treeLets);
+                treeLets[treeLetIdx].nodes[nodeIdx].setLeftChildIdx(newChildIdx);
+                OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                OPENPGL_ASSERT(node.getSplitPivot() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitPivot());
+                OPENPGL_ASSERT(newChildIdx == treeLets[treeLetIdx].nodes[nodeIdx].getLeftChildIdx());
+            } else if (treeLetLevel == 1) {
+                if(nodeIdx == 1)
+                {
+                    uint32_t childIdx = node.getLeftChildIdx();
+                    uint32_t newChildIdx = insertNode(m_nodesPtr[childIdx], 3, treeLetIdx, treeDepth+1,treeLets);
+                    insertNode(m_nodesPtr[childIdx+1], 4, treeLetIdx, treeDepth+1,treeLets);
+                    treeLets[treeLetIdx].nodes[nodeIdx].setLeftChildIdx(newChildIdx);
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    OPENPGL_ASSERT(node.getSplitPivot() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitPivot());
+                    OPENPGL_ASSERT(newChildIdx == treeLets[treeLetIdx].nodes[nodeIdx].getLeftChildIdx());
+                } else if (nodeIdx == 2) {
+                    uint32_t childIdx = node.getLeftChildIdx();
+                    uint32_t newChildIdx = insertNode(m_nodesPtr[childIdx], 5, treeLetIdx, treeDepth+1,treeLets);
+                    insertNode(m_nodesPtr[childIdx+1], 6, treeLetIdx, treeDepth+1,treeLets);
+                    treeLets[treeLetIdx].nodes[nodeIdx].setLeftChildIdx(newChildIdx);
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    OPENPGL_ASSERT(node.getSplitPivot() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitPivot());
+                    OPENPGL_ASSERT(newChildIdx == treeLets[treeLetIdx].nodes[nodeIdx].getLeftChildIdx());
+                } else {
+                    OPENPGL_ASSERT(false);
+                }
+            } else {
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    uint32_t childIdx = node.getLeftChildIdx();
+                    treeLets.push_back(KDTreeLet());
+                    treeLets.push_back(KDTreeLet());
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    uint32_t leftTreeLetIdx = treeLets.size() - 2;
+                    insertNode(m_nodesPtr[childIdx], 0, leftTreeLetIdx, treeDepth+1,treeLets);
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    insertNode(m_nodesPtr[childIdx+1], 0, leftTreeLetIdx+1, treeDepth+1,treeLets);
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    treeLets[treeLetIdx].nodes[nodeIdx].setLeftChildIdx(leftTreeLetIdx);
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+
+                    OPENPGL_ASSERT(node.getSplitDim() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitDim());
+                    OPENPGL_ASSERT(node.getSplitPivot() == treeLets[treeLetIdx].nodes[nodeIdx].getSplitPivot());
+                    OPENPGL_ASSERT(leftTreeLetIdx == treeLets[treeLetIdx].nodes[nodeIdx].getLeftChildIdx());
+            }
+        }
+        return globalNodeId;
+    }
+#ifndef USE_TREELETS
     uint32_t getDataIdxAtPos(const Vector3 &pos) const
     {
         OPENPGL_ASSERT(m_isInit);
@@ -305,15 +389,42 @@ struct KDTree
             float pivot = m_nodesPtr[nodeIdx].getSplitPivot();
 
             nodeIdx = m_nodesPtr[nodeIdx].getLeftChildIdx();
-            if (pos[splitDim] >= pivot)
-            {
-                nodeIdx++;
-            }
+            nodeIdx += pos[splitDim] >= pivot ? 1 : 0;
         }
-
         return m_nodesPtr[nodeIdx].getDataIdx();
     }
 
+#else
+    uint32_t getDataIdxAtPos(const Vector3 &pos) const
+    {
+        OPENPGL_ASSERT(m_isInit);
+        OPENPGL_ASSERT(embree::inside(m_bounds, pos));
+
+        uint32_t treeIdx = 0;
+        uint32_t nodeIdx = 0;
+        uint32_t depth = 0;
+        KDTreeLet treeLet = m_treeLets[treeIdx];
+        
+        while(!treeLet.nodes[nodeIdx].isLeaf())
+        {
+            uint8_t splitDim = treeLet.nodes[nodeIdx].getSplitDim();
+            uint32_t childIdx = treeLet.nodes[nodeIdx].getLeftChildIdx();
+            float pivot = treeLet.nodes[nodeIdx].getSplitPivot();
+            
+            if(depth % 3 == 2){
+                nodeIdx = 0;
+                treeIdx = childIdx;
+                treeIdx += pos[splitDim] >= pivot ? 1 : 0;
+                treeLet = m_treeLets[treeIdx];
+            } else {
+                nodeIdx = childIdx - (treeIdx * 8);
+                nodeIdx += pos[splitDim] >= pivot ? 1 : 0;
+            }
+            depth++;
+        }
+        return treeLet.nodes[nodeIdx].getDataIdx();
+    }
+#endif
     uint32_t getMaxNodeDepth(const KDNode& node) const
     {
         if(node.isLeaf())
@@ -475,7 +586,6 @@ struct KDTree
             objFile << "f " << vertexIDOffset+1 + 7 << "\t" << vertexIDOffset+1 + 3 << "\t" << vertexIDOffset+1 + 2 << "\t" << vertexIDOffset+1 + 6 << std::endl;
 
             vertexIDOffset += 8;
-            //std::cout << "BBox: lower = [" << sampleBound.lower[0] << ",\t"<< sampleBound.lower[1] << ",\t"<< sampleBound.lower[2] << "] \t upper = ["<< sampleBound.upper[0] << ",\t"<< sampleBound.upper[1] << ",\t"<< sampleBound.upper[2] << std::endl;
         }
     }
 
@@ -506,8 +616,64 @@ struct KDTree
             m_nodes.push_back(node);
             m_nodesPtr[n] = node;
         }
+#ifdef USE_TREELETS
+        if(num_nodes > 0)
+            buildTreeLets();
+#endif
+
     }
 
+    bool operator==(const KDTree& b) const {
+        bool equal = true;
+        if (m_isInit != b.m_isInit || m_bounds.lower.x != b.m_bounds.lower.x ||
+            m_bounds.lower.y != b.m_bounds.lower.y || m_bounds.lower.z != b.m_bounds.lower.z ||
+            m_bounds.upper.x != b.m_bounds.upper.x || m_bounds.upper.y != b.m_bounds.upper.y || 
+            m_bounds.upper.z != b.m_bounds.upper.z || 
+            m_numTreeLets != b.m_numTreeLets) 
+        {
+            equal = false;
+        }
+
+        if (m_nodes.size() == b.m_nodes.size())
+        {
+            for (int n = 0; n < m_nodes.size(); n++) {
+                if (!m_nodes[n].operator==(b.m_nodes[n])) {
+                    equal = false;
+
+                }
+            }
+        } else {
+            equal = false;
+        }
+        return equal;
+    }
+
+    int calculateMaxDepth(const KDNode& node) const 
+    {
+        if(node.isLeaf()){
+            return 1;
+        } else {
+            int childIdxLeft = node.getLeftChildIdx();
+
+            int depthLeft = calculateMaxDepth(m_nodes[childIdxLeft]);
+            int depthRight = calculateMaxDepth(m_nodes[childIdxLeft+1]);
+            return 1 + std::max(depthLeft, depthRight);
+        }
+    }
+
+    KDTreeStatistics getStatistics() const 
+    {
+        KDTreeStatistics treeStats;
+        treeStats.maxDepth = calculateMaxDepth(m_nodes[0]);
+        treeStats.numberOfNodes = m_nodes.size();
+        treeStats.sizePerNode = sizeof(KDNode);
+        treeStats.numberOfReservedNodes = m_nodes.capacity();
+        treeStats.sizeAllNodesUsed = m_nodes.size() * sizeof(KDNode);
+        treeStats.sizeAllNodesReserved = m_nodes.capacity() * sizeof(KDNode);
+        return treeStats;
+    }
+
+
 public:
     bool m_isInit { false };
 
@@ -519,6 +685,9 @@ public:
 
     // node storage used during querying
     KDNode* m_nodesPtr {nullptr};
+
+    KDTreeLet* m_treeLets {nullptr};
+    int m_numTreeLets {0};
 };
 
 }
\ No newline at end of file
